package ru.dbotthepony.kstarbound

import com.google.gson.GsonBuilder
import ru.dbotthepony.kstarbound.defs.DamageType
import ru.dbotthepony.kstarbound.defs.JsonFunction
import ru.dbotthepony.kstarbound.defs.MaterialReference
import ru.dbotthepony.kstarbound.defs.ThingDescription
import ru.dbotthepony.kstarbound.defs.image.AtlasConfiguration
import ru.dbotthepony.kstarbound.defs.image.SpriteReference
import ru.dbotthepony.kstarbound.defs.item.ArmorItemPrototype
import ru.dbotthepony.kstarbound.defs.item.CurrencyItemPrototype
import ru.dbotthepony.kstarbound.defs.item.IArmorItemDefinition
import ru.dbotthepony.kstarbound.defs.item.IFossilItemDefinition
import ru.dbotthepony.kstarbound.defs.item.IItemDefinition
import ru.dbotthepony.kstarbound.defs.item.ItemPrototype
import ru.dbotthepony.kstarbound.defs.item.ItemRarity
import ru.dbotthepony.kstarbound.defs.item.ItemTooltipKind
import ru.dbotthepony.kstarbound.defs.item.LeveledStatusEffect
import ru.dbotthepony.kstarbound.defs.item.LiquidItemPrototype
import ru.dbotthepony.kstarbound.defs.item.MaterialItemPrototype
import ru.dbotthepony.kstarbound.defs.liquid.LiquidDefinition
import ru.dbotthepony.kstarbound.defs.parallax.ParallaxPrototype
import ru.dbotthepony.kstarbound.defs.parallax.ParallaxPrototypeLayer
import ru.dbotthepony.kstarbound.defs.projectile.ActionActions
import ru.dbotthepony.kstarbound.defs.projectile.ActionConfig
import ru.dbotthepony.kstarbound.defs.projectile.ActionLoop
import ru.dbotthepony.kstarbound.defs.projectile.ActionProjectile
import ru.dbotthepony.kstarbound.defs.projectile.ActionSound
import ru.dbotthepony.kstarbound.defs.projectile.ConfigurableProjectile
import ru.dbotthepony.kstarbound.defs.projectile.ProjectilePhysics
import ru.dbotthepony.kstarbound.defs.tile.MaterialModifier
import ru.dbotthepony.kstarbound.defs.tile.RenderMatch
import ru.dbotthepony.kstarbound.defs.tile.RenderMatchList
import ru.dbotthepony.kstarbound.defs.tile.RenderParameters
import ru.dbotthepony.kstarbound.defs.tile.RenderPiece
import ru.dbotthepony.kstarbound.defs.tile.RenderRuleList
import ru.dbotthepony.kstarbound.defs.tile.RenderTemplate
import ru.dbotthepony.kstarbound.defs.tile.TileDefinition
import ru.dbotthepony.kstarbound.defs.world.SkyColoring
import ru.dbotthepony.kstarbound.defs.world.SkyColoringManifold
import ru.dbotthepony.kstarbound.defs.world.SkyParameters
import ru.dbotthepony.kstarbound.defs.world.SkySatellite
import ru.dbotthepony.kstarbound.defs.world.SkyType
import ru.dbotthepony.kstarbound.defs.world.dungeon.BeamUpRule
import ru.dbotthepony.kstarbound.defs.world.dungeon.DungeonType
import ru.dbotthepony.kstarbound.defs.world.dungeon.DungeonWorldDef
import ru.dbotthepony.kstarbound.io.ColorTypeAdapter
import ru.dbotthepony.kstarbound.io.json.AABBTypeAdapter
import ru.dbotthepony.kstarbound.io.json.AABBiTypeAdapter
import ru.dbotthepony.kstarbound.io.json.Vector2dTypeAdapter
import ru.dbotthepony.kstarbound.io.json.Vector2fTypeAdapter
import ru.dbotthepony.kstarbound.io.json.Vector2iTypeAdapter
import ru.dbotthepony.kstarbound.io.json.Vector4iTypeAdapter
import ru.dbotthepony.kstarbound.io.json.builder.EnumAdapter
import ru.dbotthepony.kstarbound.io.json.factory.ArrayListAdapterFactory
import ru.dbotthepony.kstarbound.io.json.factory.ImmutableCollectionAdapterFactory
import ru.dbotthepony.kstarbound.math.PolyTypeAdapter

fun addStarboundJsonAdapters(builder: GsonBuilder) {
	with(builder) {
		// ImmutableList, ImmutableSet, ImmutableMap
		registerTypeAdapterFactory(ImmutableCollectionAdapterFactory)

		// ArrayList
		registerTypeAdapterFactory(ArrayListAdapterFactory)

		registerTypeAdapter(ColorTypeAdapter.nullSafe())

		// математические классы
		registerTypeAdapter(AABBTypeAdapter)
		registerTypeAdapter(AABBiTypeAdapter)
		registerTypeAdapter(Vector2dTypeAdapter)
		registerTypeAdapter(Vector2fTypeAdapter)
		registerTypeAdapter(Vector2iTypeAdapter)
		registerTypeAdapter(Vector4iTypeAdapter)
		registerTypeAdapter(PolyTypeAdapter)

		// Снаряды
		registerTypeAdapterFactory(ConfigurableProjectile.ADAPTER)
		registerTypeAdapter(EnumAdapter(ProjectilePhysics::class).neverNull())
		registerTypeAdapterFactory(ActionConfig.ADAPTER)
		registerTypeAdapterFactory(ActionProjectile.ADAPTER)
		registerTypeAdapterFactory(ActionSound.ADAPTER)
		registerTypeAdapterFactory(ActionLoop.ADAPTER)
		registerTypeAdapterFactory(ActionActions.ADAPTER)

		// Параметры неба
		registerTypeAdapterFactory(SkyParameters.ADAPTER)
		registerTypeAdapter(SkyColoringManifold.ADAPTER)
		registerTypeAdapterFactory(SkyColoring.ADAPTER)
		registerTypeAdapter(EnumAdapter(SkyType::class))
		registerTypeAdapterFactory(SkySatellite.ADAPTER)
		registerTypeAdapterFactory(SkySatellite.LAYER_ADAPTER)

		// Данные о данжах
		registerTypeAdapterFactory(DungeonWorldDef.ADAPTER)
		registerTypeAdapter(EnumAdapter(BeamUpRule::class))
		registerTypeAdapter(EnumAdapter(DungeonType::class))

		// Параллакс
		registerTypeAdapterFactory(ParallaxPrototype.ADAPTER)
		registerTypeAdapterFactory(ParallaxPrototypeLayer.ADAPTER)
		registerTypeAdapter(ParallaxPrototypeLayer.LAYER_PARALLAX_ADAPTER)

		// Предметы
		registerTypeAdapterFactory(ItemPrototype.ADAPTER)
		registerTypeAdapterFactory(CurrencyItemPrototype.ADAPTER)
		registerTypeAdapterFactory(ArmorItemPrototype.ADAPTER)
		registerTypeAdapterFactory(MaterialItemPrototype.ADAPTER)
		registerTypeAdapterFactory(LiquidItemPrototype.ADAPTER)
		registerTypeAdapterFactory(IItemDefinition.InventoryIcon.ADAPTER)
		registerTypeAdapterFactory(IFossilItemDefinition.FossilSetDescription.ADAPTER)
		registerTypeAdapterFactory(IArmorItemDefinition.ArmorFrames.ADAPTER)

		// Функции
		registerTypeAdapter(JsonFunction.CONSTRAINT_ADAPTER)
		registerTypeAdapter(JsonFunction.INTERPOLATION_ADAPTER)
		registerTypeAdapter(JsonFunction.Companion)

		// Тайлы и жидкости
		registerTypeAdapterFactory(MaterialModifier.ADAPTER)
		registerTypeAdapterFactory(RenderParameters.ADAPTER)
		registerTypeAdapterFactory(RenderPiece.ADAPTER)
		registerTypeAdapterFactory(RenderRuleList.ADAPTER)
		registerTypeAdapterFactory(RenderMatch.ADAPTER)
		registerTypeAdapterFactory(RenderMatch.PIECE_ADAPTER)
		registerTypeAdapterFactory(RenderMatch.MATCHER_ADAPTER)
		registerTypeAdapterFactory(RenderMatchList.ADAPTER)
		registerTypeAdapterFactory(RenderRuleList.Entry.ADAPTER)
		registerTypeAdapterFactory(RenderTemplate.ADAPTER)
		registerTypeAdapter(EnumAdapter(RenderRuleList.Combination::class))
		registerTypeAdapterFactory(TileDefinition.ADAPTER)
		registerTypeAdapterFactory(LiquidDefinition.ADAPTER)
		registerTypeAdapterFactory(LiquidDefinition.INTERACTION_ADAPTER)

		// Общее
		registerTypeAdapter(SpriteReference.Companion)
		registerTypeAdapter(AtlasConfiguration.ADAPTER)

		registerTypeAdapterFactory(LeveledStatusEffect.ADAPTER)
		registerTypeAdapter(MaterialReference.Companion)
		registerTypeAdapter(ThingDescription.Companion)

		registerTypeAdapter(EnumAdapter(DamageType::class, default = DamageType.NORMAL))
		registerTypeAdapter(EnumAdapter(ItemRarity::class))
		registerTypeAdapter(EnumAdapter(ItemTooltipKind::class))
	}
}
