package ru.dbotthepony.kstarbound.defs.item

import com.google.common.collect.ImmutableList
import ru.dbotthepony.kstarbound.Starbound
import ru.dbotthepony.kstarbound.defs.util.enrollMap
import ru.dbotthepony.kstarbound.io.json.builder.BuilderAdapter
import ru.dbotthepony.kstarbound.io.json.util.asJsonObject
import ru.dbotthepony.kstarbound.io.json.util.asList
import ru.dbotthepony.kstarbound.io.json.neverNull
import ru.dbotthepony.kstarbound.util.NotNullVar

class ArmorItemPrototype : ItemPrototype(), IArmorItemDefinition {
	override var colorOptions: ImmutableList<Map<String, String>> = ImmutableList.of()
	override var maleFrames: IArmorItemDefinition.ArmorFrames by NotNullVar()
	override var femaleFrames: IArmorItemDefinition.ArmorFrames by NotNullVar()
	override var level: Double = 1.0
	override var leveledStatusEffects: ImmutableList<LeveledStatusEffect> = ImmutableList.of()

	override var scripts: ImmutableList<String> = ImmutableList.of()
	override var scriptDelta: Int = 1

	override var armorType: ArmorPieceType by NotNullVar()

	init {
		maxStack = 1L
	}

	override fun assemble(): IItemDefinition {
		return ArmorItemDefinition(
			descriptionData = descriptionData,
			itemName = itemName,
			price = price,
			rarity = rarity,
			category = category,
			inventoryIcon = inventoryIcon,
			itemTags = itemTags,
			learnBlueprintsOnPickup = learnBlueprintsOnPickup,
			maxStack = maxStack,
			eventCategory = eventCategory,
			consumeOnPickup = consumeOnPickup,
			pickupQuestTemplates = pickupQuestTemplates,
			scripts = scripts,
			scriptDelta = scriptDelta,
			tooltipKind = tooltipKind,
			twoHanded = twoHanded,
			radioMessagesOnPickup = radioMessagesOnPickup,
			fuelAmount = fuelAmount,

			json = enrollMap(json),

			colorOptions = colorOptions,
			maleFrames = maleFrames,
			femaleFrames = femaleFrames,
			level = level,
			leveledStatusEffects = leveledStatusEffects,

			armorType = armorType,
		)
	}

	companion object {
		val ADAPTER = BuilderAdapter.Builder(::ArmorItemPrototype)
			.also { addFields(it as BuilderAdapter.Builder<ItemPrototype>) } // безопасность: свойства родительского класса объявлены как final
			.auto(ArmorItemPrototype::colorOptions)
			.auto(ArmorItemPrototype::maleFrames)
			.auto(ArmorItemPrototype::femaleFrames)
			.auto(ArmorItemPrototype::level)
			.auto(ArmorItemPrototype::leveledStatusEffects)
			.auto(ArmorItemPrototype::scripts)
			.auto(ArmorItemPrototype::scriptDelta)
	}
}
